#!/usr/bin/perl -w
#
# Author:  Petter Reinholdtsen <pere@hungry.com>
# Date:    2002-02-04
# License: GPL
#
# To use this, run the following before this script is started:
#
#   wget http://ftp-master.debian.org/testing/update_excuses.html.gz
#   gunzip update_excuses.html.gz
#
# Then, run this script like this:
#
#   ./distdiff-testing-unstable.pl <pkglist> update_excuses.html

use vars qw($srcpkg $timestamp %stats %archstats $totalpkgs
            $debug);

$debug = 0;

my $pkglist    = shift || "pkgdeblist.txt:pkgwishlist-complete.txt";
my $excuselist = shift || "update_excuses.html";

sub load_pkg_srclist {
    my ($pkglist) = @_;

    my $file;
    for $file (split /:/, $pkglist) {
        my @list;
        print STDERR "Reading from $file\n" if $debug;
        open(PKGLIST, "<$file") || die "opening $file failed";
        chomp (@list = <PKGLIST>);
        close(PKGLIST);
        push(@pkgs, map { m/^([^_]+)/; $1 } @list);
    }

    # First add all packages to the source list, when remove the
    # individual packages if there exists a different source package.

    my ($curpkg);
    for $curpkg (@pkgs) {
        $srcpkg{$curpkg} = 1;
    }

    # Redirect STDERR to avoid getting reports on unknown packages
    open(SRCLIST, "apt-cache show @pkgs 2>&1 |")
        || die "Unable to run apt-cache";
    while (<SRCLIST>) {
        chomp;
        if (/^Package: (.+)$/) {
            $curpkg = $1;
            $srcpkg{$curpkg} = 1;
        }
        if (/^Source: (.+)$/) {
            delete $srcpkg{$curpkg};
            $srcpkg{$1} = 1;
            undef $curpkg;
        }
    }
    close(SRCLIST);

    $srcpkg{$curpkg} = 1 if (defined $curpkg);

    $totalpkgs = scalar keys %srcpkg;
}

sub init_stats {
    my $val;
    for $val (qw(buggy valid overage toyoung removing)) {
        $stats{$val} = 0;
    }
}

sub load_excuses {
    my $file = shift;

    init_stats();

    # Parse HTML and output information
    open(EXCUSES, "<$file") || die "reading $file failed";
    my ($p, $fromver, $tover, $valid_candidate, $age, $agelimit, $buggy,
        $removing);
    my ($archmissing);
    my %oldstats;
    my %oldarchstats;
    while (<EXCUSES>) {
        chomp;
        if (m%<li><a .+ name="-?(.+)">.+</a>.+\((.+) to (.+)\)%) {
            ($p, $fromver, $tover) = ($1, $2, $3);

            print STDERR "Saw package '$p'\n" if $debug;

            undef $age;
            undef $agelimit;
            $valid_candidate = "";
            $buggy = "";
            $baddepend = "";
            $depend = "";
            $removing = "";
            $archmissing = "";

            if ($tover eq "-") {
                $removing = "removing";
                $stats{"removing"}++;
            }

            %oldstats     = %stats;
            %oldarchstats = %archstats;
            #print STDERR "Copying stats $stats{buggy} and $oldstats{buggy}\n";
        }
        $timestamp = $1 if (/Generated: (.+)$/);
        if (m%Too young, only (\d+) of (\d+) day%) {
            ($age, $agelimit) = ($1, $2);
            $stats{"toyoung"}++;
        }

        if (m%(\d+) days old \(needed (\d+) day%) {
            ($age, $agelimit) = ($1, $2);
            $stats{"overage"}++;
        }

        if (m/out of date on <.+>(\S+)<.+>:/) {
            $archmissing .= " <strike><a href=\"http://www.buildd.net/cgi/package_status?$1_pkg=$p&searchtype=go\">$1</a></strike>";
            $archstats{$1}++;
        }

        if (/>buggy</) {
            $buggy = "buggy";
            $stats{"buggy"}++;
        }

        if (/Trying to remove package, not update it/) {
            $removing = "removing";
            $stats{"removing"}++;
        }

        if (/unsatisfiable Depends/) {
            $baddepend = "baddepend";
            # $stats{"baddepend"}++;
        }

        if (/>Depends: /) {
            $depend = "depend";
            # $stats{"depend"}++;
        }

        if (m%<li>Valid candidate%) {
            $valid_candidate = "<a href=\"http://bjorn.haxx.se/debian/testing.pl?package=$p\">valid</a>";
            $stats{"valid"}++;
        }

        if ( m%</ul>%i) {
            if ($srcpkg{$p} ) {
                $stats{"total"}++;
                push(@pkginfo, [$p,$fromver,$tover, $valid_candidate, $age,
                                $agelimit, $buggy, $archmissing, $baddepend,
                                $depend, $removing]);
            } else {
                # print STDERR " + restoring old stats $stats{buggy} and $oldstats{buggy}\n";

                # Throw away changes to the statistic.  We do not care
                # about the changes
                %stats     = %oldstats     if %oldstats;
                %archstats = %oldarchstats if %oldarchstats;

            }
        }
    }
    close(EXCUSES);
}

sub print_pkginfo {
    my ($srcpkg, $fromver, $tover, $valid_candidate, $age, $agelimit,
      $buggy, $archmissing, $baddepend, $depend, $removing) = @_;
#    print "  $srcpkg $fromver $tover\n";

    $excusepkg = $srcpkg;
    $excusepkg = "-$srcpkg" if "-" eq $tover;

    my $ageinfo = "";
    if (defined $age) {;
        if ($agelimit < $age) {
            $ageinfo = "<strong>$age/$agelimit</strong>";
        } else {
            $ageinfo = "$age/$agelimit";
        }
    }

    my $flags = "$valid_candidate $baddepend $depend $buggy $removing $archmissing";
    $flags =~ s/^\s*//;
    $flags =~ s/\s*$//;
    $flags =~ s/\s*/ /;

    print <<EOF;
 <tr>
  <th align="left">$srcpkg</th>
  <td width="15ex">$flags&nbsp;</td>
  <td>$ageinfo&nbsp;</td>
  <td>$fromver</td>
  <td>$tover</td>
  <td>
    <a href="http://packages.qa.debian.org/$srcpkg">overview</a> |
    <a href="http://buildd.debian.org/build.php?pkg=$srcpkg">buildd</a> |
    <a href="http://bugs.debian.org/cgi-bin/pkgreport.cgi?src=$srcpkg">bugs</a></td>
 </tr>
EOF
#  <td>@pkgs</td>
}

load_pkg_srclist($pkglist);
load_excuses($excuselist);

print <<EOF;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<title>Skolelinux: Package diff bethween Debian testing and unstable</title>
</head>
<body>
<h1>Package diff bethween Debian testing and unstable</h1>

  <p>Comparing a list of packages in Debian testing and unstable, to
     see which packages need help to get the newer versions into
     testing.  Package with bugs, but no new version with bugfixes in
     unstable is not shown here.<p>

<h2>Statistics</h2>
<ul>

<li>$totalpkgs packages total.
<li>$stats{total} packages with differences.
<li>$stats{valid} valid packages.
<li>$stats{buggy} buggy packages.
<li>$stats{overage} packages over age.
<li>$stats{toyoung} packages to young.
<li>$stats{removing} packages marked for removal.
<li>Missing archiectures blocking packages:
EOF
my $arch;
for $arch (sort { $archstats{$b} <=> $archstats{$a} }
           keys %archstats) {
    print "  $arch($archstats{$arch})\n";
}

print <<EOF;
</ul>

Based on
<a href="http://ftp-master.debian.org/testing/update_excuses.html.gz">$excuselist</a>
generated $timestamp.

<table border=1>
<tr><th>Package name</th><th>Flags</th><th>Days</th>
<th>From version</th><th>To version</th><th>Links</th>
</tr>
EOF

sub pkgsort {
    # Sort on days since last upload
    my $retval = (@{$b}[4] || -1) <=> (@{$a}[4] || -1);
    # Or age limit if that fails
    $retval = (@{$b}[5] || -1) <=> (@{$a}[5] || -1) if (0 == $retval);
    # Or name if that fails
    $retval = (@{$b}[0] cmp @{$a}[0]) if (0 == $retval);
    return $retval;
}

my $info;
for $info (sort pkgsort @pkginfo) {
        print_pkginfo(@{$info});
}

print <<EOF;
</table>

EOF


print <<EOF;
<h2>Flags</h2>
<dl>

<dt>valid</dt>
<dd>The package is in princible accepted for inclusion, but adding it to
Woody would remove some other packages.  Not sure how to solve this, as it
is unknown which package is blocking the move.</dd>

<dt>buggy</dt>
<dd>The package in unstable have more critical, grave or serious bugs than
the version in testing.  These bugs must be fixed (closed) before the version
will move into testing.</dd>

<dt>baddepend</dt>
<dd>The package in unstable needs, for one or more platforms, some
packages (or versions of packages) which are still missing form
testing.</dd>

<dt>depend</dt>
<dd>The package in unstable needs a package which are still missing form
testing.  Not quite sure about the difference between baddepend and
depend.</dd>

<dt>removing</dt>
<dd>The package is marked for removal from Testing.  It will be removed as
soon as the packages depending on it is removed as well.</dd>

<dt><strike>architecture</strike></dt>
<dd>The package is missing binaries for the listed architecture, and
this block the package transition into testing.  This is probably due
to some compile error.  Check the buildd log and report a bug if this
is a source error.</dd>

</dl>

<p>More info on how the packages are moved from unstable to testing is
available from <a href="http://www.linuxmafia.com/debian/testingfaq.html">The
Unofficial Debian Testing FAQ</a> and
<a href="http://www.debian.org/devel/testing">'a couple of notes' on the
testing distribution</a>.</p>

<p>This summary was generated by $0, available from
<a href="http://developer.skolelinux.no/cgi-bin/viewcvs.cgi/skolelinux/src/task-skolelinux/tools/distdiff-testing-unstable.pl">viewcvs</a>.</p>

</body></html>
EOF

